@@ -23,8 +23,8 @@ var playbackColor = '#a0729fcf' /* playback blue */
 var baseColor = '#2b2b2b'
 var offlineColor = '#4b4b4b'
 
-var hasCloseButton = true
-var hasTaskSwitcher = true
+var hasCloseButton = false
+var hasTaskSwitcher = false
 
 var listItemBigText = listItemHeight * .35
 var listItemSmallText = listItemHeight * .25
