@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,9 +18,10 @@
 #
 
 __author__    = 'Thomas Perl <thp@gpodder.org>'
-__version__   = '3.0.2'
-__date__      = '2011-12-13'
-__copyright__ = '© 2005-2011 Thomas Perl and the gPodder Team'
+__version__   = '3.0.4'
+__date__      = '2012-01-24'
+__relname__   = 'Weekend Vampire'
+__copyright__ = '© 2005-2012 Thomas Perl and the gPodder Team'
 __license__   = 'GNU General Public License, version 3 or later'
 __url__       = 'http://gpodder.org/'
 
@@ -88,8 +89,16 @@ osx = (platform.system() == 'Darwin')
 textdomain = 'gpodder'
 locale_dir = gettext.bindtextdomain(textdomain)
 t = gettext.translation(textdomain, locale_dir, fallback=True)
-gettext = t.ugettext
-ngettext = t.ungettext
+
+try:
+    # Python 2
+    gettext = t.ugettext
+    ngettext = t.ungettext
+except AttributeError:
+    # Python 3
+    gettext = t.gettext
+    ngettext = t.ngettext
+
 if win32:
     try:
         # Workaround for bug 650
