@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@ logger = logging.getLogger(__name__)
 
 from gpodder import util
 from gpodder import youtube
+from gpodder import vimeo
 import gpodder
 
 import threading
@@ -45,9 +46,9 @@ import collections
 
 import mimetypes
 import email
-import email.Header
 
-import cgi
+from email.header import decode_header
+
 
 _ = gpodder.gettext
 
@@ -68,7 +69,7 @@ def get_header_param(headers, param, header_name):
             value = msg.get_param(param, header=header_name)
             if value is None:
                 return None
-            decoded_list = email.Header.decode_header(value)
+            decoded_list = decode_header(value)
             value = []
             for part, encoding in decoded_list:
                 if encoding:
@@ -471,6 +472,7 @@ class DownloadTask(object):
         task.status_changed   # True if the status has been changed (see below)
         task.url              # URL of the episode being downloaded
         task.podcast_url      # URL of the podcast this download belongs to
+        task.episode          # Episode object of this task
 
     You can cancel a running download task by setting its status:
 
@@ -581,10 +583,6 @@ class DownloadTask(object):
         self.__episode = episode
         self._config = config
 
-        # Set names for the downloads list
-        self.markup_name = cgi.escape(self.__episode.title)
-        self.markup_podcast_name = cgi.escape(self.__episode.channel.title)
-
         # Create the target filename and save it in the database
         self.filename = self.__episode.local_filename(create=True)
         self.tempname = self.filename + '.partial'
@@ -735,6 +733,8 @@ class DownloadTask(object):
             # Resolve URL and start downloading the episode
             url = youtube.get_real_download_url(self.__episode.url, \
                     self._config.youtube_preferred_fmt_id)
+            url = vimeo.get_real_download_url(url)
+
             downloader =  DownloadURLOpener(self.__episode.channel)
             headers, real_url = downloader.retrieve_resume(url, \
                     self.tempname, reporthook=self.status_updated)
