@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -130,8 +130,10 @@ class CoverDownloader(ObservableService):
             return (channel.url, self.get_default_cover(channel))
 
         if not os.path.exists(channel.cover_file):
-            if url is None:
-                url = channel.cover_url
+            if url is None and channel.cover_url is not None:
+                # We have to use authenticate_url, because password-protected
+                # feeds might keep their cover art also protected (bug 1521)
+                url = channel.authenticate_url(channel.cover_url)
 
             new_url = youtube.get_real_cover(channel.url)
             if new_url is not None:
