@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -59,6 +59,7 @@ if not hasattr(mygpoclient, 'require_version') or \
     sys.exit(1)
 
 from mygpoclient import api
+from mygpoclient import public
 
 from mygpoclient import util as mygpoutil
 
@@ -588,3 +589,18 @@ class MygPoClient(object):
     def open_website(self):
         util.open_website('http://' + self._config.mygpo_server)
 
+
+class Directory(object):
+    def __init__(self):
+        self.client = public.PublicClient()
+
+    def toplist(self):
+        return [(p.title or p.url, p.url)
+                for p in self.client.get_toplist()
+                if p.url]
+
+    def search(self, query):
+        return [(p.title or p.url, p.url)
+                for p in self.client.search_podcasts(query)
+                if p.url]
+
