@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -65,7 +65,7 @@ def get_param(s, param='filename', header='content-disposition'):
     msg = email.message_from_string(s)
     if header in msg:
         value = msg.get_param(param, header=header)
-        decoded_list = email.Header.decode_header(value)
+        decoded_list = email.header.decode_header(value)
         value = []
         for part, encoding in decoded_list:
             if encoding:
