@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 #
 
 
-from PySide.QtCore import QObject, Property, Signal
+from PySide.QtCore import QObject, Property, Signal, Slot
 
 import gpodder
 
@@ -37,14 +37,7 @@ from gpodder import model
 import threading
 import os
 
-def convert(s):
-    if s is None:
-        return None
-
-    if isinstance(s, unicode):
-        return s
-
-    return s.decode('utf-8', 'ignore')
+convert = util.convert_bytes
 
 class QEpisode(QObject):
     def __init__(self, wrapper_manager, podcast, episode):
@@ -65,6 +58,11 @@ class QEpisode(QObject):
         # Playback tracking
         self._qt_playing = False
 
+    @Slot(QObject, result=bool)
+    def equals(self, other):
+        """Needed for Python object identity comparison in JavaScript"""
+        return self == other
+
     def __getattr__(self, name):
         logger.warn('Attribute access in %s: %s', self.__class__.__name__, name)
         return getattr(self._episode, name)
@@ -227,6 +225,11 @@ class QPodcast(QObject):
         self._podcast = podcast
         self._updating = False
 
+    @Slot(QObject, result=bool)
+    def equals(self, other):
+        """Needed for Python object identity comparison in JavaScript"""
+        return self == other
+
     @classmethod
     def sort_key(cls, qpodcast):
         if isinstance(qpodcast, cls):
@@ -241,6 +244,10 @@ class QPodcast(QObject):
         return getattr(self._podcast, name)
 
     def qupdate(self, force=False, finished_callback=None):
+        if self._updating:
+            # Update in progress - just wait, don't re-update
+            return
+
         def t(self):
             self._updating = True
             self.changed.emit()
