@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -56,6 +56,8 @@ class Matcher(object):
             return episode.state == gpodder.STATE_DELETED
         elif k == 'played':
             return not episode.is_new
+        elif k == 'downloading':
+            return episode.downloading
         elif k == 'archive':
             return episode.archive
         elif k in ('finished', 'fin'):
