@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
 # Copyright (c) 2011 Neal H. Walfield
 #
 # gPodder is free software; you can redistribute it and/or modify
@@ -237,19 +237,17 @@ def username_password_from_url(url):
       ...
     ValueError: URL has to be a string or unicode object.
     >>> username_password_from_url('http://a@b:c@host.com/')
-    Traceback (most recent call last):
-      ...
-    ValueError: "@" must be encoded for username/password (RFC1738).
+    ('a@b', 'c')
     >>> username_password_from_url('ftp://a:b:c@host.com/')
-    Traceback (most recent call last):
-      ...
-    ValueError: ":" must be encoded for username/password (RFC1738).
+    ('a', 'b:c')
     >>> username_password_from_url('http://i%2Fo:P%40ss%3A@host.com/')
     ('i/o', 'P@ss:')
     >>> username_password_from_url('ftp://%C3%B6sterreich@host.com/')
     ('\xc3\xb6sterreich', None)
     >>> username_password_from_url('http://w%20x:y%20z@example.org/')
     ('w x', 'y z')
+    >>> username_password_from_url('http://example.com/x@y:z@test.com/')
+    (None, None)
     """
     if type(url) not in (str, unicode):
         raise ValueError('URL has to be a string or unicode object.')
@@ -262,11 +260,20 @@ def username_password_from_url(url):
         (authentication, netloc) = netloc.rsplit('@', 1)
         if ':' in authentication:
             (username, password) = authentication.split(':', 1)
-            # RFC1738 dictates that we should not allow these unquoted
-            # characters in the username and password field (Section 3.1).
-            for c in (':', '@', '/'):
-                if c in username or c in password:
-                    raise ValueError('"%c" must be encoded for username/password (RFC1738).' % c)
+
+            # RFC1738 dictates that we should not allow ['/', '@', ':']
+            # characters in the username and password field (Section 3.1):
+            #
+            # 1. The "/" can't be in there at this point because of the way
+            #    urlparse (which we use above) works.
+            # 2. Due to gPodder bug 1521, we allow "@" in the username and
+            #    password field. We use netloc.rsplit('@', 1), which will
+            #    make sure that we split it at the last '@' in netloc.
+            # 3. The colon must be excluded (RFC2617, Section 2) in the
+            #    username, but is apparently allowed in the password. This
+            #    is handled by the authentication.split(':', 1) above, and
+            #    will cause any extraneous ':'s to be part of the password.
+
             username = urllib.unquote(username)
             password = urllib.unquote(password)
         else:
@@ -829,13 +836,15 @@ def url_strip_authentication(url):
     'http://x.org/'
     >>> url_strip_authentication('http://P%40%3A:i%2F@cx.lan')
     'http://cx.lan'
+    >>> url_strip_authentication('http://x@x.com:s3cret@example.com/')
+    'http://example.com/'
     """
     url_parts = list(urlparse.urlsplit(url))
     # url_parts[1] is the HOST part of the URL
 
     # Remove existing authentication data
     if '@' in url_parts[1]:
-        url_parts[1] = url_parts[1].split('@', 2)[1]
+        url_parts[1] = url_parts[1].rsplit('@', 1)[1]
 
     return urlparse.urlunsplit(url_parts)
 
@@ -858,21 +867,24 @@ def url_add_authentication(url, username, password):
     >>> url_add_authentication('http://localhost/x', 'aa', 'bc')
     'http://aa:bc@localhost/x'
     >>> url_add_authentication('http://blubb.lan/u.html', 'i/o', 'P@ss:')
-    'http://i%2Fo:P%40ss%3A@blubb.lan/u.html'
+    'http://i%2Fo:P@ss:@blubb.lan/u.html'
     >>> url_add_authentication('http://a:b@x.org/', 'c', 'd')
     'http://c:d@x.org/'
-    >>> url_add_authentication('http://i%2F:P%40%3A@cx.lan', 'P@:', 'i/')
-    'http://P%40%3A:i%2F@cx.lan'
+    >>> url_add_authentication('http://i%2F:P%40%3A@cx.lan', 'P@x', 'i/')
+    'http://P@x:i%2F@cx.lan'
     >>> url_add_authentication('http://x.org/', 'a b', 'c d')
     'http://a%20b:c%20d@x.org/'
     """
     if username is None or username == '':
         return url
 
-    username = urllib.quote(username, safe='')
+    # Relaxations of the strict quoting rules (bug 1521):
+    # 1. Accept '@' in username and password
+    # 2. Acecpt ':' in password only
+    username = urllib.quote(username, safe='@')
 
     if password is not None:
-        password = urllib.quote(password, safe='')
+        password = urllib.quote(password, safe='@:')
         auth_string = ':'.join((username, password))
     else:
         auth_string = username
@@ -1170,6 +1182,35 @@ def open_website(url):
     else:
         threading.Thread(target=webbrowser.open, args=(url,)).start()
 
+def convert_bytes(d):
+    """
+    Convert byte strings to unicode strings
+
+    This function will decode byte strings into unicode
+    strings. Any other data types will be left alone.
+
+    >>> convert_bytes(None)
+    >>> convert_bytes(1)
+    1
+    >>> convert_bytes(4711L)
+    4711L
+    >>> convert_bytes(True)
+    True
+    >>> convert_bytes(3.1415)
+    3.1415
+    >>> convert_bytes('Hello')
+    u'Hello'
+    >>> convert_bytes(u'Hey')
+    u'Hey'
+    """
+    if d is None:
+        return d
+    if any(isinstance(d, t) for t in (int, long, bool, float)):
+        return d
+    elif not isinstance(d, unicode):
+        return d.decode('utf-8', 'ignore')
+    return d
+
 def sanitize_encoding(filename):
     r"""
     Generate a sanitized version of a string (i.e.
@@ -1278,12 +1319,16 @@ def find_mount_point(directory):
     >>> restore()
     """
     if isinstance(directory, unicode):
+        # XXX: This is only valid for Python 2 - misleading error in Python 3?
         # We do not accept unicode strings, because they could fail when
         # trying to be converted to some native encoding, so fail loudly
         # and leave it up to the callee to encode into the proper encoding.
         raise ValueError('Convert unicode objects to str first.')
 
     if not isinstance(directory, str):
+        # In Python 2, we assume it's a byte str; in Python 3, we assume
+        # that it's a unicode str. The abspath/ismount/split functions of
+        # os.path work with unicode str in Python 3, but not in Python 2.
         raise ValueError('Directory names should be of type str.')
 
     directory = os.path.abspath(directory)
