@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -146,6 +146,9 @@ def get_youtube_id(url):
 def is_video_link(url):
     return (get_youtube_id(url) is not None)
 
+def is_youtube_guid(guid):
+    return guid.startswith('tag:youtube.com,2008:video:')
+
 def get_real_channel_url(url):
     r = re.compile('http://(?:[a-z]+\.)?youtube\.com/user/([a-z0-9]+)', re.IGNORECASE)
     m = r.match(url)
